#!/usr/bin/env node
'use strict';

const fs = require('fs');
const Crypt = require('..');

const USAGE = `
Usage: jcrypt <enc|dec> <passphrase> <inputPath> <outputPath>
`;

function printUsage() {
    console.log(USAGE);
}

function parseArgs() {
    var args = process.argv.slice(2);
    return ['command', 'passphrase', 'input', 'output'].reduce((ps, p) => {
        let value = args.shift();
        if (!value) {
            printUsage();
            process.exit(1);
        }
        ps[p] = value;
        return ps;
    }, {});
}

function main() {
    let params = parseArgs();
    switch (params.command.toLowerCase()) {
        case 'enc':
            return fs.readFile(params.input, {encoding: 'utf8'}, (err, data) => {
                if (err) return console.error(err);
                Crypt.dump(JSON.parse(data), params.passphrase, params.output);
            });
        case 'dec':
            return Crypt.load(params.passphrase, params.input)
                        .then(object =>  fs.writeFile(params.output, JSON.stringify(object)));
        default:
            printUsage();
            process.exit(1);
    }
}

main();