'use strict';

const crypto = require('crypto');
const fs = require('fs');
const Buffer = require('buffer').Buffer;

const AES = 'aes-256-cbc';
const ITERATIONS = 100000;
const KEY_LEN = 32;
const DIGEST_HASH = 'sha256';
const UTF8 = 'utf8';

function dump(object, passphrase, filePath) {
    return new Promise((resolve, reject) => {
        crypto.randomBytes(KEY_LEN * 1.5, (err, buf) => {
            if (err) reject(err);
            let salt = buf.slice(0, KEY_LEN);
            crypto.pbkdf2(passphrase, salt, ITERATIONS, KEY_LEN, DIGEST_HASH, (err, key) => {
                if (err) reject(err);                
                let iv = buf.slice(KEY_LEN);
                let cipherStream = crypto.createCipheriv(AES, key, iv);
                let encryptedBuffer = cipherStream.update(JSON.stringify(object), UTF8);
                fs.writeFile(filePath, 
                             Buffer.concat([buf, encryptedBuffer, cipherStream.final()]), 
                             {encoding: null}, 
                             err => err ? reject(err) : resolve());
            });
        });
    });
}

function dumpSync(object, passphrase, filePath) {
    let buf = crypto.randomBytes(KEY_LEN * 1.5);
    let salt = buf.slice(0, KEY_LEN);
    let key = crypto.pbkdf2Sync(passphrase, salt, ITERATIONS, KEY_LEN, DIGEST_HASH);
    let iv = buf.slice(KEY_LEN);
    let cipherStream = crypto.createCipheriv(AES, key, iv);
    let encryptedBuffer = cipherStream.update(JSON.stringify(object), UTF8);
    fs.writeFileSync(filePath, 
                     Buffer.concat([buf, encryptedBuffer, cipherStream.final()]), 
                     {encoding: null});
}

function load(passphrase, filePath) {
    return new Promise((resolve, reject) => {
        fs.readFile(filePath, (err, data) => {
            if (err) reject(err);
            let buf = data.slice(0, KEY_LEN * 1.5);
            let encData = data.slice(KEY_LEN * 1.5);
            let salt = buf.slice(0, KEY_LEN);
            let iv = buf.slice(KEY_LEN);
            crypto.pbkdf2(passphrase, salt, ITERATIONS, KEY_LEN, DIGEST_HASH, (err, key) => {
                if (err) reject(err);                        
                let decipherStream = crypto.createDecipheriv(AES, key, iv);
                let plain = decipherStream.update(encData, null, UTF8);
                plain += decipherStream.final(UTF8);
                resolve(JSON.parse(plain));
            });
        });
    });
}

function loadSync(passphrase, filePath) {
    let data = fs.readFileSync(filePath);
    let buf = data.slice(0, KEY_LEN * 1.5);
    let encData = data.slice(KEY_LEN * 1.5);
    let salt = buf.slice(0, KEY_LEN);
    let iv = buf.slice(KEY_LEN);
    let key = crypto.pbkdf2Sync(passphrase, salt, ITERATIONS, KEY_LEN, DIGEST_HASH);
    let decipherStream = crypto.createDecipheriv(AES, key, iv);
    let plain = decipherStream.update(encData, null, UTF8);
    plain += decipherStream.final(UTF8);
    return JSON.parse(plain);
}

module.exports = {
    dump,
    dumpSync,
    load,
    loadSync
};